-- WELCOME TO MÖAN - Minimal smööthing Audio Manager
--
--
--
--


moan = {}

-- loads or appends sound info to already known sound base
-- path - default sound path, if not specified uses already saved path in möan.
--		defaults to current directory
-- what - table of sound names and and their loading/playing info
--		should be in format: (** denotes optional args)
--			sound_name = {
--				filename = "name_of_audio_file",
--				**how = "static" | "stream" - how sound should be loaded to play
--				(default: 'stream')
--				**path = "path_to_sound", overrides default path
function moan.load(what, path, preload)
	local s = s or {}
	moan.path = path or moan.path or '.'
	if what and preload then
		for k,v in pairs(what) do
			if not v.filename then
				print("warning: no filename information for entry.. " .. k)
			elseif not love.filesystem.exists((v.path or moan.path) .. v.filename) then
				print("warning: file not found.. " .. (v.path or moan.path)..v.filename)
			else
				s[k] = love.audio.newSource((v.path or path)..v.filename, v.how)
			end
		end
	end
	moan._src = s
	if moan._info then
		for k,v in pairs(what) do
			moan._info[k] = v
		end
	else
		moan._info = what
	end
	moan._kills = moan._kills or {}
end

-- resets everything, safe to use load afterwards
function moan.reset()
	for k,v in pairs(moan._src) do
		v:stop()
	end
	moan._src = nil
	moan._kills = nil
	moan._info = nil
	moan.path = nil
end
	

-- MUST: call on every update, frees up memory, enables smooth transitions
function moan.update(dt)
	-- smooth transitions
	local krem = {}
	for k,v in pairs(moan._kills) do
		if moan._src[k] then
			local vol = moan._src[k]:getVolume()
			if v.time > 0 then
				watcher.watch(11, v)
				v.time = v.time - dt
				local pro =(v.time/v.otime)
				vol = (v.from-v.to)* pro + v.to
				moan._src[k]:setVolume(vol)
			elseif not v.nokill then
				moan._src[k]:stop()
				table.insert(krem, k)
			else
				table.insert(krem, k)
			end
		end
	end

	-- src'es management
	local rem = {}
	for k,v in pairs(moan._src) do
		if v:isStopped() then
			table.insert(rem, k)
			if moan._kills[k] then
				table.insert(krem, k)
			end
		end
	end
	for k,v in ipairs(rem) do
		if moan._kills[v] then
			moan._kills[v] = nil
		end
	end
	for k,v in ipairs(rem) do
		moan._src[k] = nil
	end
end

-- smooth sound kill over time
-- what - name of sound
-- speed - in what time to kill
-- from - start at which volume lvl
-- to - at what volume lvl to kill
function moan.kill(what, speed, from, to, nokill)
	if moan._src[what] and not moan._src[what]:isStopped() then
		local k = {
			time = speed,
			otime = speed,
			from = 'c' == from and moan._src[what]:getVolume() or from or 1,
			to = to or 0,
			nokill = nokill,
		}
		moan._kills[what] = k
	end
end

-- gets or set volume of specified sound
function moan.volume(what, set)
	if set == nil then
		return moan._src[what]:getVolume()
	else
		moan._src[what]:setVolume(set)
	end
end


-- plays sound specified by 'what' name
-- loop - sets looping to true (default: false)
-- how - specifies whenever to 'stream' or 'static' (default: stream), overrides
--	values in inition table
function moan.play(what, loop, how)
	local n = moan._src[what]
	if not n then
		local ni = moan._info[what] or {}
		if not ni.filename then
			print("warning: no filename information for entry.. " .. what)
			return
		elseif not love.filesystem.exists((ni.path or moan.path) .. ni.filename) then
			print("warning: file not found.. " .. (ni.path or moan.path)..ni.filename)
			return
		end
		n = love.audio.newSource((ni.path or moan.path)..ni.filename, (how or ni.how))
		moan._src[what] = n
	end
	n:setLooping(loop)
	n:play()
	return n
end


-- stops sound specified sound from playing
function moan.stop(what)
	if moan._src[what] then
		moan._src[what]:stop()
		moan._src[what] = nil
	end
	if moan._kills[what] then
		moan._kills[what] = nil
	end
end
