char_const = {
	speed = 100,
	jumppwr = -50,
	xmove = 20,
}
local const = char_const

local STATES = {
	run = 'run',
	jump = "jump",
	idle = "idle",
	attack = "attack",
	drop = "drop",
	run_h = 'run_h',
	run_up = 'run_up',
	run_down = 'run_down',
	idle_down = 'idle_down',
	idle_up = 'idle_up',
	idle_h = 'idle_h',
}
STATES.downgrade = {
		['run_down'] = STATES.idle_down,
		['run_up'] = STATES.idle_up,
		['run_h'] = STATES.idle_h,
	}



local function init(me, infoname, time)
	me.lvltime = time
	me.shape = vector(32, 32)
	if infoname then
		me.images = {}
		me.info = love.filesystem.load(infoname..'.lua')()
		for k,v in pairs(me.info) do
			local i = love.graphics.newImage(img..v.filename)
			me.images[k] = i
			local g = anim8.newGrid(v.dim.x, v.dim.y, i:getWidth(), i:getHeight())
			me.info[k].grid = g
			me.info[k].anim = anim8.newAnimation(v.tipe, g(v.frames), v.delay, v.delays)
			me.info[k].anim:pause()
		end
		me.astate = STATES.idle_down
	end
	me.info[me.astate].anim:resume()
	return me
end

local function draw(me)
	if me.info then
		me.info[me.astate].anim:draw(me.images[me.astate], me.pos.x, me.pos.y)
	else
		love.graphics.rectangle("line", me.pos.x, me.pos.y, me.shape.x, me.shape.y)
	end
end

local function update(me, dt)
	if me.mult > 0 and me.lvlno == '7' and (me.spawn-me.pos):len() > 40 then
		me.mult = -2
	end
	me.opos = me.pos:clone()
	if not me.timedelay then
		me.lvltime = me.lvltime - dt
	end
	me.action = nil
	if me.info then
		me.info[me.astate].anim:update(dt)
	end
	if me.timedelay and not me.last then
		me.timedelay = me.timedelay - dt
		if me.timedelay < 0 then
			me.nexter()
		end
	elseif me.train.stopped then
		if love.keyboard.isDown('d') then
			me.pos.x = me.pos.x + const.speed*me.mult*dt
		end
		if love.keyboard.isDown('a') then
			me.pos.x = me.pos.x - const.speed*me.mult*dt
		end
		if love.keyboard.isDown('w') then
			me.pos.y = me.pos.y - const.speed*me.mult*dt
		end
		if love.keyboard.isDown('s') then
			me.pos.y = me.pos.y + const.speed*me.mult*dt
		end
		local diff = me.pos - me.opos
		local nstate
		local stop, flip = false, false
		local moving = false
		if diff.x ~= 0 then
			nstate = STATES.run_h
			moving = true
			if diff.x > 0 then
				if me.info[nstate].anim.flippedH then
					flip = true
				end
				me.info[nstate].anim.flippedH = false
			else
				if not me.info[nstate].anim.flippedH then
					flip = true
				end
				me.info[nstate].anim.flippedH = true
			end
		else
			if diff.y > 0 then
				nstate = STATES.run_down
				moving = true
			elseif diff.y < 0 then
				nstate = STATES.run_up
				moving = true
			elseif not flip then
				nstate = me.astate
				stop = true
			end
		end
		if moving then
			if sounds.step[sounds.step.curr]:tell() > 0.6 then
				sounds.step[sounds.step.curr]:stop()
				sounds.step.curr = (sounds.step.curr + #sounds.step) % (#sounds.step ) + 1
			end
			sounds.step[sounds.step.curr]:play()
		end
		if nstate ~= me.astate or flip then
			me.info[me.astate].anim:pause()
			me.info[me.astate].anim:gotoFrame(1)
			me.info[nstate].anim:resume()
			me.astate = nstate
		elseif stop then
			me.info[me.astate].anim:pause()
			me.info[me.astate].anim:gotoFrame(1)
		end

		if love.keyboard.isDown('j') then
			me.action = true
		end
	end
		
end

local function collect(me)
	me.stone = true
end

local function keypressed(me, key, uni)

end

local function keyreleased(me, key, uni)

end

local function reset(me)

end

local function winDelayer(me, time, nexter, strer)
	me.timedelay = time
	me.nexter = nexter
	me.train:timer(time)
	--starttrain
end


function newRunner(x, y)
	return setmetatable( {
		mult = 1,
		winDelayer = winDelayer,
		off = vector(8, 2),
		pos = vector(x, y),
		spawn = vector(x, y),
		vel = vector(),
		init = init,
		update = update,
		draw = draw,
		reset = reset,
		collect = collect,
		keypressed = keypressed,
		keyreleased = keyreleased,
	}, {
	} )
end

return setmetatable({}, {__call = function(_, ...) return newRunner(...) end} )

