local STATES = {
	run = "run",
	idle = "idle",
}

local const = {
	prex = -500,
	prey = -80,
	x = -100,
	y = -80,
	stationx = 500,
	endpoint = 800,
	soundend = 600,
	speed = 150,
}
	


local function update(me, dt, stor)
	local moved = (me.pos.x-const.prex)/(const.x-const.prex)
	if moved < 1 then
		me.pos.x = me.pos.x + dt*const.speed*(1-moved+0.2)
	elseif not me.stopped and moved > 0.8 then
		me.pos.x = const.x
		me.astate = STATES.idle
		moan.kill('train', 0.5, 'c')
		me.stopped = true
	end

	if me.time then
		me.time = me.time - dt
		me.pos.x = me.pos.x + dt*const.speed
		me.astate = STATES.run
		me.info[me.astate].anim:resume()
	end
	if me.pos.x > const.soundend then
		moan.kill("train", 1, 'c')
	end
	if me.pos.x > const.endpoint then
		stor:remove(me)
	end
	me.info[me.astate].anim:update(dt)
end

local function special_update(me, dt, stor)
	local pro
	if me.time then
		me.time = me.time - dt
		pro = me.time / me.otime
		if pro > 0.05 then
			me.pos.x = const.prex + (const.stationx-const.prex)*(1-math.pow((pro), 2))
			me.info[me.astate].anim:resume()
		else
			me.astate = STATES.idle
			me.info[me.astate].anim:resume()
			me.stopped = true
		end
	end
	if me.pos.x > const.endpoint then
		stor:remove(me)
	end
	if me.pos.x > const.soundend*4/5 then
		moan.kill("train", 1, 'c')
	end
	if pro > 0.05 then
		me.info[me.astate].anim:update(dt)
	end
end

local function intro_up(me, dt, stor)
	local moved = (me.pos.x-const.prex)/(const.stationx/2-const.prex)
	if moved < 1 then
		me.pos.x = me.pos.x + dt*const.speed*(1-moved+0.5)
	elseif not me.stopped and moved > 0.8 then
		me.pos.x = const.stationx/2
		me.stopped = true
	end

	if me.time then
		if me.time < 0 then
			me.intro_end = true
		end
		me.time = me.time - dt
		local t = (1-me.time/me.otime) * (const.endpoint-const.stationx/2)
		me.pos.x = const.stationx/2+ t
		me.astate = STATES.run
		me.info[me.astate].anim:resume()
	end
	me.info[me.astate].anim:update(dt)
end

local function draw(me)
	me.info[me.astate].anim:draw(me.images[me.astate], me.pos.x, me.pos.y)
end

local function setTime(me, time)
	me.time = time
	me.otime = time
	moan.play('train')
	moan.kill("train", 0.2, 0, 1, true)
end



local function init(me, special)
	me.update = '11' == special and special_update or me.update
	me.update = 'intro' == special and intro_up or me.update
	me.drawhi = true
	moan.play('train', true)
	moan.kill("train", 0.2, 0, 1, true)
	me.images = {}
	me.info = love.filesystem.load( "traininfo"..'.lua')()
	for k,v in pairs(me.info) do
		local i = love.graphics.newImage(img..v.filename)
		me.images[k] = i
		local g = anim8.newGrid(v.dim.x, v.dim.y, i:getWidth(), i:getHeight())
		me.info[k].grid = g
		me.info[k].anim = anim8.newAnimation(v.tipe, g(v.frames), v.delay, v.delays)
		me.info[k].anim:pause()
	end
	me.astate = STATES.run
	me.info[me.astate].anim:resume()
	return me
end
	

function newTrain(time)
	return setmetatable( {
		pos = vector(const.prex, const.prey),
		init = init,
		timer = setTime,
		update = update,
		draw = draw,
		time = time,
		otime = time,
	}, {} )
end


return setmetatable({}, {__call = function(_, ...) return newTrain(...) end} )
		
